use std::sync::OnceLock;
use error_stack::Report;
use futures_executor::block_on;
use rs_firebase_admin_sdk::{
    auth::{FirebaseAuthService, UserIdentifiers},
    client::ApiHttpClient,
    App, AuthenticationManager,
};
use rs_firebase_admin_sdk::auth::token::{LiveTokenVerifier, TokenVerifier};
use rs_firebase_admin_sdk::auth::token::cache::{HttpCache, PubKeys};
use rs_firebase_admin_sdk::auth::token::error::TokenVerificationError;
use rs_firebase_admin_sdk::auth::token::jwt::JWToken;

static LIVE_APP: OnceLock<App<AuthenticationManager>> = OnceLock::new();

static ID_TOKEN_VERIFIER: OnceLock<LiveTokenVerifier<HttpCache<reqwest::Client, PubKeys>>> = OnceLock::new();

fn get_app() -> &'static App<AuthenticationManager> {
    LIVE_APP.get_or_init(|| block_on(async {
        let gcp_service_account = AuthenticationManager::new().await.expect("创建authentication manager失败");
        App::live(gcp_service_account.into()).await.expect("创建live app失败")
    }))
}

fn get_id_token_verifier() -> &'static LiveTokenVerifier<HttpCache<reqwest::Client, PubKeys>> {
    ID_TOKEN_VERIFIER.get_or_init(|| block_on(async {
        get_app().id_token_verifier().await.expect("创建id token verifier失败")
    }))
}

pub async fn verify_user_id_token(id_token: &str) -> Result<JWToken, String> {
    match get_id_token_verifier().verify_token(id_token).await {
        Ok(jwtToken) => {
            Ok(jwtToken)
        }
        Err(error) => {
            tracing::error!("校验id token失败: {:?}", error);
            Err(error.to_string())
        }
    }
}

async fn test() {
    // Load your GCP SA from env, see https://crates.io/crates/gcp_auth for more details
    let gcp_service_account = AuthenticationManager::new().await.unwrap();
    // Create live (not emulated) context for Firebase app
    let live_app = App::live(gcp_service_account.into()).await.unwrap();

    let id_token_verifier = live_app.id_token_verifier().await.expect("Error while creating id token verifier");
    let verify_result = id_token_verifier.verify_token("").await.expect("Error while verifying token");
    // Create Firebase authentication admin client
    let auth_admin = live_app.auth();

    let user = auth_admin.get_user(
        // Build a filter for finding the user
        UserIdentifiers::builder()
        .with_email("me@email.com".into())
        .build()
    )
        .await
        .expect("Error while fetching user")
        .expect("User does not exist");

    println!("User id: {}", user.uid);
}